# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.client_credential import ClientCredential
from ...types.client_credential_type_enum import ClientCredentialTypeEnum
from ...types.get_client_credential_response_content import GetClientCredentialResponseContent
from ...types.patch_client_credential_response_content import PatchClientCredentialResponseContent
from ...types.post_client_credential_response_content import PostClientCredentialResponseContent
from ...types.public_key_credential_algorithm_enum import PublicKeyCredentialAlgorithmEnum
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCredentialsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, client_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[ClientCredential]]:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[ClientCredential]]
            Credentials successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ClientCredential],
                    parse_obj_as(
                        type_=typing.List[ClientCredential],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        client_id: str,
        *,
        credential_type: ClientCredentialTypeEnum,
        name: typing.Optional[str] = OMIT,
        subject_dn: typing.Optional[str] = OMIT,
        pem: typing.Optional[str] = OMIT,
        alg: typing.Optional[PublicKeyCredentialAlgorithmEnum] = OMIT,
        parse_expiry_from_cert: typing.Optional[bool] = OMIT,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        kid: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostClientCredentialResponseContent]:
        """
        Create a client credential associated to your application. Credentials can be used to configure Private Key JWT and mTLS authentication methods, as well as for JWT-secured Authorization requests.

        **Public Key**

        Public Key credentials can be used to set up Private Key JWT client authentication and JWT-secured Authorization requests.

        Sample:

        ```json
        {
          "credential_type": "public_key",
          "name": "string",
          "pem": "string",
          "alg": "RS256",
          "parse_expiry_from_cert": false,
          "expires_at": "2022-12-31T23:59:59Z"
        }
        ```

        **Certificate (CA-signed & self-signed)**

        Certificate credentials can be used to set up mTLS client authentication. CA-signed certificates can be configured either with a signed certificate or with just the certificate Subject DN.

        CA-signed Certificate Sample (pem):

        ```json
        {
          "credential_type": "x509_cert",
          "name": "string",
          "pem": "string"
        }
        ```

        CA-signed Certificate Sample (subject_dn):

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "subject_dn": "string"
        }
        ```

        Self-signed Certificate Sample:

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "pem": "string"
        }
        ```

        The credential will be created but not yet enabled for use until you set the corresponding properties in the client:

        - To enable the credential for Private Key JWT or mTLS authentication methods, set the `client_authentication_methods` property on the client. For more information, read [Configure Private Key JWT Authentication](https://auth0.com/docs/get-started/applications/configure-private-key-jwt) and [Configure mTLS Authentication](https://auth0.com/docs/get-started/applications/configure-mtls)
        - To enable the credential for JWT-secured Authorization requests, set the `signed_request_object`property on the client. For more information, read [Configure JWT-secured Authorization Requests (JAR)](https://auth0.com/docs/get-started/applications/configure-jar)

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_type : ClientCredentialTypeEnum

        name : typing.Optional[str]
            Friendly name for a credential.

        subject_dn : typing.Optional[str]
            Subject Distinguished Name. Mutually exclusive with `pem` property. Applies to `cert_subject_dn` credential type.

        pem : typing.Optional[str]
            PEM-formatted public key (SPKI and PKCS1) or X509 certificate. Must be JSON escaped.

        alg : typing.Optional[PublicKeyCredentialAlgorithmEnum]

        parse_expiry_from_cert : typing.Optional[bool]
            Parse expiry from x509 certificate. If true, attempts to parse the expiry date from the provided PEM. Applies to `public_key` credential type.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential. If not specified (not recommended), the credential never expires. Applies to `public_key` credential type.

        kid : typing.Optional[str]
            Optional kid (Key ID), used to uniquely identify the credential. If not specified, a kid value will be auto-generated. The kid header parameter in JWTs sent by your client should match this value. Valid format is [0-9a-zA-Z-_]{10,64}

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostClientCredentialResponseContent]
            Credential successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials",
            method="POST",
            json={
                "credential_type": credential_type,
                "name": name,
                "subject_dn": subject_dn,
                "pem": pem,
                "alg": alg,
                "parse_expiry_from_cert": parse_expiry_from_cert,
                "expires_at": expires_at,
                "kid": kid,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostClientCredentialResponseContent,
                    parse_obj_as(
                        type_=PostClientCredentialResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetClientCredentialResponseContent]:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetClientCredentialResponseContent]
            Credential successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials/{encode_path_param(credential_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetClientCredentialResponseContent,
                    parse_obj_as(
                        type_=GetClientCredentialResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Delete a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials/{encode_path_param(credential_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        client_id: str,
        credential_id: str,
        *,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PatchClientCredentialResponseContent]:
        """
        Change a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PatchClientCredentialResponseContent]
            Credential successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials/{encode_path_param(credential_id)}",
            method="PATCH",
            json={
                "expires_at": expires_at,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PatchClientCredentialResponseContent,
                    parse_obj_as(
                        type_=PatchClientCredentialResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCredentialsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, client_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[ClientCredential]]:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[ClientCredential]]
            Credentials successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[ClientCredential],
                    parse_obj_as(
                        type_=typing.List[ClientCredential],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        client_id: str,
        *,
        credential_type: ClientCredentialTypeEnum,
        name: typing.Optional[str] = OMIT,
        subject_dn: typing.Optional[str] = OMIT,
        pem: typing.Optional[str] = OMIT,
        alg: typing.Optional[PublicKeyCredentialAlgorithmEnum] = OMIT,
        parse_expiry_from_cert: typing.Optional[bool] = OMIT,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        kid: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostClientCredentialResponseContent]:
        """
        Create a client credential associated to your application. Credentials can be used to configure Private Key JWT and mTLS authentication methods, as well as for JWT-secured Authorization requests.

        **Public Key**

        Public Key credentials can be used to set up Private Key JWT client authentication and JWT-secured Authorization requests.

        Sample:

        ```json
        {
          "credential_type": "public_key",
          "name": "string",
          "pem": "string",
          "alg": "RS256",
          "parse_expiry_from_cert": false,
          "expires_at": "2022-12-31T23:59:59Z"
        }
        ```

        **Certificate (CA-signed & self-signed)**

        Certificate credentials can be used to set up mTLS client authentication. CA-signed certificates can be configured either with a signed certificate or with just the certificate Subject DN.

        CA-signed Certificate Sample (pem):

        ```json
        {
          "credential_type": "x509_cert",
          "name": "string",
          "pem": "string"
        }
        ```

        CA-signed Certificate Sample (subject_dn):

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "subject_dn": "string"
        }
        ```

        Self-signed Certificate Sample:

        ```json
        {
          "credential_type": "cert_subject_dn",
          "name": "string",
          "pem": "string"
        }
        ```

        The credential will be created but not yet enabled for use until you set the corresponding properties in the client:

        - To enable the credential for Private Key JWT or mTLS authentication methods, set the `client_authentication_methods` property on the client. For more information, read [Configure Private Key JWT Authentication](https://auth0.com/docs/get-started/applications/configure-private-key-jwt) and [Configure mTLS Authentication](https://auth0.com/docs/get-started/applications/configure-mtls)
        - To enable the credential for JWT-secured Authorization requests, set the `signed_request_object`property on the client. For more information, read [Configure JWT-secured Authorization Requests (JAR)](https://auth0.com/docs/get-started/applications/configure-jar)

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_type : ClientCredentialTypeEnum

        name : typing.Optional[str]
            Friendly name for a credential.

        subject_dn : typing.Optional[str]
            Subject Distinguished Name. Mutually exclusive with `pem` property. Applies to `cert_subject_dn` credential type.

        pem : typing.Optional[str]
            PEM-formatted public key (SPKI and PKCS1) or X509 certificate. Must be JSON escaped.

        alg : typing.Optional[PublicKeyCredentialAlgorithmEnum]

        parse_expiry_from_cert : typing.Optional[bool]
            Parse expiry from x509 certificate. If true, attempts to parse the expiry date from the provided PEM. Applies to `public_key` credential type.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential. If not specified (not recommended), the credential never expires. Applies to `public_key` credential type.

        kid : typing.Optional[str]
            Optional kid (Key ID), used to uniquely identify the credential. If not specified, a kid value will be auto-generated. The kid header parameter in JWTs sent by your client should match this value. Valid format is [0-9a-zA-Z-_]{10,64}

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostClientCredentialResponseContent]
            Credential successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials",
            method="POST",
            json={
                "credential_type": credential_type,
                "name": name,
                "subject_dn": subject_dn,
                "pem": pem,
                "alg": alg,
                "parse_expiry_from_cert": parse_expiry_from_cert,
                "expires_at": expires_at,
                "kid": kid,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostClientCredentialResponseContent,
                    parse_obj_as(
                        type_=PostClientCredentialResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetClientCredentialResponseContent]:
        """
        Get the details of a client credential.

        **Important**: To enable credentials to be used for a client authentication method, set the `client_authentication_methods` property on the client. To enable credentials to be used for JWT-Secured Authorization requests set the `signed_request_object` property on the client.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetClientCredentialResponseContent]
            Credential successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials/{encode_path_param(credential_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetClientCredentialResponseContent,
                    parse_obj_as(
                        type_=GetClientCredentialResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, client_id: str, credential_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials/{encode_path_param(credential_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        client_id: str,
        credential_id: str,
        *,
        expires_at: typing.Optional[dt.datetime] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PatchClientCredentialResponseContent]:
        """
        Change a client credential you previously created. May be enabled or disabled. For more information, read <a href="https://www.auth0.com/docs/get-started/authentication-and-authorization-flow/client-credentials-flow">Client Credential Flow</a>.

        Parameters
        ----------
        client_id : str
            ID of the client.

        credential_id : str
            ID of the credential.

        expires_at : typing.Optional[dt.datetime]
            The ISO 8601 formatted date representing the expiration of the credential.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PatchClientCredentialResponseContent]
            Credential successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"clients/{encode_path_param(client_id)}/credentials/{encode_path_param(credential_id)}",
            method="PATCH",
            json={
                "expires_at": expires_at,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PatchClientCredentialResponseContent,
                    parse_obj_as(
                        type_=PatchClientCredentialResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .branding_theme_font_body_text import BrandingThemeFontBodyText
from .branding_theme_font_buttons_text import BrandingThemeFontButtonsText
from .branding_theme_font_input_labels import BrandingThemeFontInputLabels
from .branding_theme_font_links import BrandingThemeFontLinks
from .branding_theme_font_links_style_enum import BrandingThemeFontLinksStyleEnum
from .branding_theme_font_subtitle import BrandingThemeFontSubtitle
from .branding_theme_font_title import BrandingThemeFontTitle


class BrandingThemeFonts(UniversalBaseModel):
    body_text: BrandingThemeFontBodyText
    buttons_text: BrandingThemeFontButtonsText
    font_url: str = pydantic.Field()
    """
    Font URL
    """

    input_labels: BrandingThemeFontInputLabels
    links: BrandingThemeFontLinks
    links_style: BrandingThemeFontLinksStyleEnum
    reference_text_size: float = pydantic.Field()
    """
    Reference text size
    """

    subtitle: BrandingThemeFontSubtitle
    title: BrandingThemeFontTitle

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.create_export_users_fields import CreateExportUsersFields
from ...types.create_export_users_response_content import CreateExportUsersResponseContent
from ...types.job_file_format_enum import JobFileFormatEnum
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawUsersExportsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        connection_id: typing.Optional[str] = OMIT,
        format: typing.Optional[JobFileFormatEnum] = OMIT,
        limit: typing.Optional[int] = OMIT,
        fields: typing.Optional[typing.Sequence[CreateExportUsersFields]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateExportUsersResponseContent]:
        """
        Export all users to a file via a long-running job.

        Parameters
        ----------
        connection_id : typing.Optional[str]
            connection_id of the connection from which users will be exported.

        format : typing.Optional[JobFileFormatEnum]

        limit : typing.Optional[int]
            Limit the number of records.

        fields : typing.Optional[typing.Sequence[CreateExportUsersFields]]
            List of fields to be included in the CSV. Defaults to a predefined set of fields.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateExportUsersResponseContent]
            Job created successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            "jobs/users-exports",
            method="POST",
            json={
                "connection_id": connection_id,
                "format": format,
                "limit": limit,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[CreateExportUsersFields], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateExportUsersResponseContent,
                    parse_obj_as(
                        type_=CreateExportUsersResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawUsersExportsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        connection_id: typing.Optional[str] = OMIT,
        format: typing.Optional[JobFileFormatEnum] = OMIT,
        limit: typing.Optional[int] = OMIT,
        fields: typing.Optional[typing.Sequence[CreateExportUsersFields]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateExportUsersResponseContent]:
        """
        Export all users to a file via a long-running job.

        Parameters
        ----------
        connection_id : typing.Optional[str]
            connection_id of the connection from which users will be exported.

        format : typing.Optional[JobFileFormatEnum]

        limit : typing.Optional[int]
            Limit the number of records.

        fields : typing.Optional[typing.Sequence[CreateExportUsersFields]]
            List of fields to be included in the CSV. Defaults to a predefined set of fields.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateExportUsersResponseContent]
            Job created successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "jobs/users-exports",
            method="POST",
            json={
                "connection_id": connection_id,
                "format": format,
                "limit": limit,
                "fields": convert_and_respect_annotation_metadata(
                    object_=fields, annotation=typing.Sequence[CreateExportUsersFields], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateExportUsersResponseContent,
                    parse_obj_as(
                        type_=CreateExportUsersResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionUpdatedObject2OptionsSignatureAlgorithmEnum = typing.Union[
    typing.Literal["rsa-sha1", "rsa-sha256"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

ActionExecutionStatusEnum = typing.Union[
    typing.Literal["unspecified", "pending", "final", "partial", "canceled", "suspended"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.get_risk_assessments_settings_new_device_response_content import (
    GetRiskAssessmentsSettingsNewDeviceResponseContent,
)
from ....types.update_risk_assessments_settings_new_device_response_content import (
    UpdateRiskAssessmentsSettingsNewDeviceResponseContent,
)
from .raw_client import AsyncRawNewDeviceClient, RawNewDeviceClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NewDeviceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawNewDeviceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawNewDeviceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawNewDeviceClient
        """
        return self._raw_client

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetRiskAssessmentsSettingsNewDeviceResponseContent:
        """
        Gets the risk assessment settings for the new device assessor

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetRiskAssessmentsSettingsNewDeviceResponseContent
            Returning risk assessment settings for new devices

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.risk_assessments.settings.new_device.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self, *, remember_for: int, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateRiskAssessmentsSettingsNewDeviceResponseContent:
        """
        Updates the risk assessment settings for the new device assessor

        Parameters
        ----------
        remember_for : int
            Length of time to remember devices for, in days.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateRiskAssessmentsSettingsNewDeviceResponseContent
            Returning risk assessment settings for new devices

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.risk_assessments.settings.new_device.update(
            remember_for=1,
        )
        """
        _response = self._raw_client.update(remember_for=remember_for, request_options=request_options)
        return _response.data


class AsyncNewDeviceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawNewDeviceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawNewDeviceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawNewDeviceClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetRiskAssessmentsSettingsNewDeviceResponseContent:
        """
        Gets the risk assessment settings for the new device assessor

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetRiskAssessmentsSettingsNewDeviceResponseContent
            Returning risk assessment settings for new devices

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.risk_assessments.settings.new_device.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self, *, remember_for: int, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateRiskAssessmentsSettingsNewDeviceResponseContent:
        """
        Updates the risk assessment settings for the new device assessor

        Parameters
        ----------
        remember_for : int
            Length of time to remember devices for, in days.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateRiskAssessmentsSettingsNewDeviceResponseContent
            Returning risk assessment settings for new devices

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.risk_assessments.settings.new_device.update(
                remember_for=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(remember_for=remember_for, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_binding_ref import ActionBindingRef
from .action_secret_request import ActionSecretRequest


class ActionBindingWithRef(UniversalBaseModel):
    ref: ActionBindingRef
    display_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the binding.
    """

    secrets: typing.Optional[typing.List[ActionSecretRequest]] = pydantic.Field(default=None)
    """
    The list of secrets that are included in an action or a version of an action.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .brute_force_protection_mode_enum import BruteForceProtectionModeEnum
from .brute_force_protection_shields_enum import BruteForceProtectionShieldsEnum


class GetBruteForceSettingsResponseContent(UniversalBaseModel):
    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether or not brute force attack protections are active.
    """

    shields: typing.Optional[typing.List[BruteForceProtectionShieldsEnum]] = pydantic.Field(default=None)
    """
    Action to take when a brute force protection threshold is violated.
            Possible values: <code>block</code>, <code>user_notification</code>.
    """

    allowlist: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of trusted IP addresses that will not have attack protection enforced against them.
    """

    mode: typing.Optional[BruteForceProtectionModeEnum] = None
    max_attempts: typing.Optional[int] = pydantic.Field(default=None)
    """
    Maximum number of unsuccessful attempts.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_sha_1_thumbprint import ConnectionSha1Thumbprint

ConnectionThumbprintsAd = typing.List[ConnectionSha1Thumbprint]
"""
Array of certificate SHA-1 thumbprints for validating signatures. Managed by Auth0 when using the AD Connector agent.
"""



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.create_custom_domain_response_content import CreateCustomDomainResponseContent
from ..types.custom_domain_custom_client_ip_header import CustomDomainCustomClientIpHeader
from ..types.custom_domain_provisioning_type_enum import CustomDomainProvisioningTypeEnum
from ..types.custom_domain_tls_policy_enum import CustomDomainTlsPolicyEnum
from ..types.custom_domain_verification_method_enum import CustomDomainVerificationMethodEnum
from ..types.domain_metadata import DomainMetadata
from ..types.get_custom_domain_response_content import GetCustomDomainResponseContent
from ..types.get_default_domain_response_content import GetDefaultDomainResponseContent
from ..types.list_custom_domains_response_content import ListCustomDomainsResponseContent
from ..types.test_custom_domain_response_content import TestCustomDomainResponseContent
from ..types.update_custom_domain_response_content import UpdateCustomDomainResponseContent
from ..types.update_default_domain_response_content import UpdateDefaultDomainResponseContent
from ..types.verify_custom_domain_response_content import VerifyCustomDomainResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCustomDomainsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        q: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListCustomDomainsResponseContent]:
        """
        Retrieve details on [custom domains](https://auth0.com/docs/custom-domains).

        Parameters
        ----------
        q : typing.Optional[str]
            Query in <a href ="https://lucene.apache.org/core/2_9_4/queryparsersyntax.html">Lucene query string syntax</a>.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        sort : typing.Optional[str]
            Field to sort by. Only <code>domain:1</code> (ascending order by domain) is supported at this time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListCustomDomainsResponseContent]
            Custom domains successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "custom-domains",
            method="GET",
            params={
                "q": q,
                "fields": fields,
                "include_fields": include_fields,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCustomDomainsResponseContent,
                    parse_obj_as(
                        type_=ListCustomDomainsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        domain: str,
        type: CustomDomainProvisioningTypeEnum,
        verification_method: typing.Optional[CustomDomainVerificationMethodEnum] = OMIT,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateCustomDomainResponseContent]:
        """
        Create a new custom domain.

        Note: The custom domain will need to be verified before it will accept
        requests.

        Optional attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Parameters
        ----------
        domain : str
            Domain name.

        type : CustomDomainProvisioningTypeEnum

        verification_method : typing.Optional[CustomDomainVerificationMethodEnum]

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. If not provided, the full domain will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateCustomDomainResponseContent]
            Custom domain successfully created (verification is pending).
        """
        _response = self._client_wrapper.httpx_client.request(
            "custom-domains",
            method="POST",
            json={
                "domain": domain,
                "type": type,
                "verification_method": verification_method,
                "tls_policy": tls_policy,
                "custom_client_ip_header": custom_client_ip_header,
                "domain_metadata": domain_metadata,
                "relying_party_identifier": relying_party_identifier,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCustomDomainResponseContent,
                    parse_obj_as(
                        type_=CreateCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetDefaultDomainResponseContent]:
        """
        Retrieve the tenant's default domain.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetDefaultDomainResponseContent]
            Default domain successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "custom-domains/default",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDefaultDomainResponseContent,
                    parse_obj_as(
                        type_=GetDefaultDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def set_default(
        self, *, domain: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[UpdateDefaultDomainResponseContent]:
        """
        Set the default custom domain for the tenant.

        Parameters
        ----------
        domain : str
            The domain to set as the default custom domain. Must be a verified custom domain or the canonical domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateDefaultDomainResponseContent]
            Default custom domain set successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            "custom-domains/default",
            method="PATCH",
            json={
                "domain": domain,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateDefaultDomainResponseContent,
                    parse_obj_as(
                        type_=UpdateDefaultDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetCustomDomainResponseContent]:
        """
        Retrieve a custom domain configuration and status.

        Parameters
        ----------
        id : str
            ID of the custom domain to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetCustomDomainResponseContent]
            Custom domain successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCustomDomainResponseContent,
                    parse_obj_as(
                        type_=GetCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete a custom domain and stop serving requests for it.

        Parameters
        ----------
        id : str
            ID of the custom domain to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateCustomDomainResponseContent]:
        """
        Update a custom domain.

        These are the attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        **Updating CUSTOM_CLIENT_IP_HEADER for a custom domain**

        To update the `custom_client_ip_header` for a domain, the body to
        send should be:

        ```json
        { "custom_client_ip_header": "cf-connecting-ip" }
        ```

        **Updating TLS_POLICY for a custom domain**

        To update the `tls_policy` for a domain, the body to send should be:

        ```json
        { "tls_policy": "recommended" }
        ```

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Some considerations:

        - The TLS ciphers and protocols available in each TLS policy follow industry recommendations, and may be updated occasionally.
        - The `compatible` TLS policy is no longer supported.

        Parameters
        ----------
        id : str
            The id of the custom domain to update

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]
            recommended includes TLS 1.2

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. Set to null to remove the rpId and fall back to using the full domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateCustomDomainResponseContent]
            Custom domain updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}",
            method="PATCH",
            json={
                "tls_policy": tls_policy,
                "custom_client_ip_header": custom_client_ip_header,
                "domain_metadata": domain_metadata,
                "relying_party_identifier": relying_party_identifier,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateCustomDomainResponseContent,
                    parse_obj_as(
                        type_=UpdateCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def test(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TestCustomDomainResponseContent]:
        """
        Run the test process on a custom domain.

        Parameters
        ----------
        id : str
            ID of the custom domain to test.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TestCustomDomainResponseContent]
            Custom domain test successfully completed.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}/test",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestCustomDomainResponseContent,
                    parse_obj_as(
                        type_=TestCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def verify(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[VerifyCustomDomainResponseContent]:
        """
        Run the verification process on a custom domain.

        Note: Check the `status` field to see its verification status. Once verification is complete, it may take up to 10 minutes before the custom domain can start accepting requests.

        For `self_managed_certs`, when the custom domain is verified for the first time, the response will also include the `cname_api_key` which you will need to configure your proxy. This key must be kept secret, and is used to validate the proxy requests.

        [Learn more](https://auth0.com/docs/custom-domains#step-2-verify-ownership) about verifying custom domains that use Auth0 Managed certificates.
        [Learn more](https://auth0.com/docs/custom-domains/self-managed-certificates#step-2-verify-ownership) about verifying custom domains that use Self Managed certificates.

        Parameters
        ----------
        id : str
            ID of the custom domain to verify.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VerifyCustomDomainResponseContent]
            Custom domain successfully verified.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}/verify",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VerifyCustomDomainResponseContent,
                    parse_obj_as(
                        type_=VerifyCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawCustomDomainsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        q: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        sort: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListCustomDomainsResponseContent]:
        """
        Retrieve details on [custom domains](https://auth0.com/docs/custom-domains).

        Parameters
        ----------
        q : typing.Optional[str]
            Query in <a href ="https://lucene.apache.org/core/2_9_4/queryparsersyntax.html">Lucene query string syntax</a>.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        sort : typing.Optional[str]
            Field to sort by. Only <code>domain:1</code> (ascending order by domain) is supported at this time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListCustomDomainsResponseContent]
            Custom domains successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "custom-domains",
            method="GET",
            params={
                "q": q,
                "fields": fields,
                "include_fields": include_fields,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListCustomDomainsResponseContent,
                    parse_obj_as(
                        type_=ListCustomDomainsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        domain: str,
        type: CustomDomainProvisioningTypeEnum,
        verification_method: typing.Optional[CustomDomainVerificationMethodEnum] = OMIT,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateCustomDomainResponseContent]:
        """
        Create a new custom domain.

        Note: The custom domain will need to be verified before it will accept
        requests.

        Optional attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Parameters
        ----------
        domain : str
            Domain name.

        type : CustomDomainProvisioningTypeEnum

        verification_method : typing.Optional[CustomDomainVerificationMethodEnum]

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. If not provided, the full domain will be used.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateCustomDomainResponseContent]
            Custom domain successfully created (verification is pending).
        """
        _response = await self._client_wrapper.httpx_client.request(
            "custom-domains",
            method="POST",
            json={
                "domain": domain,
                "type": type,
                "verification_method": verification_method,
                "tls_policy": tls_policy,
                "custom_client_ip_header": custom_client_ip_header,
                "domain_metadata": domain_metadata,
                "relying_party_identifier": relying_party_identifier,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCustomDomainResponseContent,
                    parse_obj_as(
                        type_=CreateCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetDefaultDomainResponseContent]:
        """
        Retrieve the tenant's default domain.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetDefaultDomainResponseContent]
            Default domain successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "custom-domains/default",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetDefaultDomainResponseContent,
                    parse_obj_as(
                        type_=GetDefaultDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def set_default(
        self, *, domain: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[UpdateDefaultDomainResponseContent]:
        """
        Set the default custom domain for the tenant.

        Parameters
        ----------
        domain : str
            The domain to set as the default custom domain. Must be a verified custom domain or the canonical domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateDefaultDomainResponseContent]
            Default custom domain set successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "custom-domains/default",
            method="PATCH",
            json={
                "domain": domain,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateDefaultDomainResponseContent,
                    parse_obj_as(
                        type_=UpdateDefaultDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetCustomDomainResponseContent]:
        """
        Retrieve a custom domain configuration and status.

        Parameters
        ----------
        id : str
            ID of the custom domain to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetCustomDomainResponseContent]
            Custom domain successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetCustomDomainResponseContent,
                    parse_obj_as(
                        type_=GetCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete a custom domain and stop serving requests for it.

        Parameters
        ----------
        id : str
            ID of the custom domain to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        tls_policy: typing.Optional[CustomDomainTlsPolicyEnum] = OMIT,
        custom_client_ip_header: typing.Optional[CustomDomainCustomClientIpHeader] = OMIT,
        domain_metadata: typing.Optional[DomainMetadata] = OMIT,
        relying_party_identifier: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateCustomDomainResponseContent]:
        """
        Update a custom domain.

        These are the attributes that can be updated:

        - custom_client_ip_header
        - tls_policy

        **Updating CUSTOM_CLIENT_IP_HEADER for a custom domain**

        To update the `custom_client_ip_header` for a domain, the body to
        send should be:

        ```json
        { "custom_client_ip_header": "cf-connecting-ip" }
        ```

        **Updating TLS_POLICY for a custom domain**

        To update the `tls_policy` for a domain, the body to send should be:

        ```json
        { "tls_policy": "recommended" }
        ```

        TLS Policies:

        - recommended - for modern usage this includes TLS 1.2 only

        Some considerations:

        - The TLS ciphers and protocols available in each TLS policy follow industry recommendations, and may be updated occasionally.
        - The `compatible` TLS policy is no longer supported.

        Parameters
        ----------
        id : str
            The id of the custom domain to update

        tls_policy : typing.Optional[CustomDomainTlsPolicyEnum]
            recommended includes TLS 1.2

        custom_client_ip_header : typing.Optional[CustomDomainCustomClientIpHeader]

        domain_metadata : typing.Optional[DomainMetadata]

        relying_party_identifier : typing.Optional[str]
            Relying Party ID (rpId) to be used for Passkeys on this custom domain. Set to null to remove the rpId and fall back to using the full domain.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateCustomDomainResponseContent]
            Custom domain updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}",
            method="PATCH",
            json={
                "tls_policy": tls_policy,
                "custom_client_ip_header": custom_client_ip_header,
                "domain_metadata": domain_metadata,
                "relying_party_identifier": relying_party_identifier,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateCustomDomainResponseContent,
                    parse_obj_as(
                        type_=UpdateCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def test(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TestCustomDomainResponseContent]:
        """
        Run the test process on a custom domain.

        Parameters
        ----------
        id : str
            ID of the custom domain to test.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TestCustomDomainResponseContent]
            Custom domain test successfully completed.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}/test",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    TestCustomDomainResponseContent,
                    parse_obj_as(
                        type_=TestCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def verify(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[VerifyCustomDomainResponseContent]:
        """
        Run the verification process on a custom domain.

        Note: Check the `status` field to see its verification status. Once verification is complete, it may take up to 10 minutes before the custom domain can start accepting requests.

        For `self_managed_certs`, when the custom domain is verified for the first time, the response will also include the `cname_api_key` which you will need to configure your proxy. This key must be kept secret, and is used to validate the proxy requests.

        [Learn more](https://auth0.com/docs/custom-domains#step-2-verify-ownership) about verifying custom domains that use Auth0 Managed certificates.
        [Learn more](https://auth0.com/docs/custom-domains/self-managed-certificates#step-2-verify-ownership) about verifying custom domains that use Self Managed certificates.

        Parameters
        ----------
        id : str
            ID of the custom domain to verify.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VerifyCustomDomainResponseContent]
            Custom domain successfully verified.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"custom-domains/{encode_path_param(id)}/verify",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VerifyCustomDomainResponseContent,
                    parse_obj_as(
                        type_=VerifyCustomDomainResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.create_event_stream_action_request_content import CreateEventStreamActionRequestContent
from ...types.create_event_stream_event_bridge_request_content import CreateEventStreamEventBridgeRequestContent
from ...types.create_event_stream_web_hook_request_content import CreateEventStreamWebHookRequestContent

EventStreamsCreateRequest = typing.Union[
    CreateEventStreamWebHookRequestContent,
    CreateEventStreamEventBridgeRequestContent,
    CreateEventStreamActionRequestContent,
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionsDiscoveryUrl = typing.Optional[str]
"""
URL of the identity provider's OIDC Discovery endpoint (/.well-known/openid-configuration). When provided and oidc_metadata is empty, Auth0 automatically retrieves the provider's configuration including endpoints and supported features. Only applicable when strategy=oidc, okta, or samlp.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .connection_profile_strategy_overrides_connection_config import ConnectionProfileStrategyOverridesConnectionConfig
from .connection_profile_strategy_overrides_enabled_features import ConnectionProfileStrategyOverridesEnabledFeatures


class ConnectionProfileStrategyOverride(UniversalBaseModel):
    """
    Connection Profile Strategy Override
    """

    enabled_features: typing.Optional[ConnectionProfileStrategyOverridesEnabledFeatures] = None
    connection_config: typing.Optional[ConnectionProfileStrategyOverridesConnectionConfig] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .user_attribute_profile_oidc_mapping import UserAttributeProfileOidcMapping
from .user_attribute_profile_saml_mapping import UserAttributeProfileSamlMapping
from .user_attribute_profile_strategy_overrides import UserAttributeProfileStrategyOverrides


class UserAttributeProfileUserAttributeAdditionalProperties(UniversalBaseModel):
    description: str = pydantic.Field()
    """
    Description of this attribute
    """

    label: str = pydantic.Field()
    """
    Display label for this attribute
    """

    profile_required: bool = pydantic.Field()
    """
    Whether this attribute is required in the profile
    """

    auth_0_mapping: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="auth0_mapping"),
        pydantic.Field(alias="auth0_mapping", description="Auth0 mapping for this attribute"),
    ]
    """
    Auth0 mapping for this attribute
    """

    oidc_mapping: typing.Optional[UserAttributeProfileOidcMapping] = None
    saml_mapping: typing.Optional[UserAttributeProfileSamlMapping] = pydantic.Field(default=None)
    """
    SAML mapping for this attribute
    """

    scim_mapping: typing.Optional[str] = pydantic.Field(default=None)
    """
    SCIM mapping for this attribute
    """

    strategy_overrides: typing.Optional[UserAttributeProfileStrategyOverrides] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_255 import ConnectionHttpsUrlWithHttpFallback255

ConnectionUserAuthorizationUrloAuth1 = ConnectionHttpsUrlWithHttpFallback255
"""
The URL of the OAuth 1.0a user-authorization endpoint. This endpoint is used to redirect users to the provider's site to authorize the temporary request token obtained from the request-token endpoint during the OAuth 1.0a authentication flow.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_context_request_geo import EventStreamCloudEventContextRequestGeo


class EventStreamCloudEventContextRequest(UniversalBaseModel):
    """
    An HTTP request.
    """

    geo: EventStreamCloudEventContextRequestGeo
    hostname: str = pydantic.Field()
    """
    The hostname the request is for.
    """

    custom_domain: typing.Optional[str] = pydantic.Field(default=None)
    """
    The custom domain used in the request (if any).
    """

    ip: str = pydantic.Field()
    """
    The originating IP address of the request.
    """

    method: str = pydantic.Field()
    """
    The HTTP method used for the request.
    """

    user_agent: str = pydantic.Field()
    """
    The value of the `User-Agent` header.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .event_stream_cloud_event_user_deleted_object_identities_item_passwordless_is_social_enum import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessIsSocialEnum,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_passwordless_profile_data import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessProfileData,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_passwordless_provider_enum import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessProviderEnum,
)
from .event_stream_cloud_event_user_deleted_object_identities_item_passwordless_user_id import (
    EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessUserId,
)


class EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordless(UniversalBaseModel):
    """
    The identity object for passwordless identity providers.
    """

    connection: str = pydantic.Field()
    """
    Name of the connection containing this identity.
    """

    user_id: EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessUserId
    profile_data: typing_extensions.Annotated[
        typing.Optional[EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessProfileData],
        FieldMetadata(alias="profileData"),
        pydantic.Field(alias="profileData"),
    ] = None
    provider: EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessProviderEnum
    is_social: typing_extensions.Annotated[
        EventStreamCloudEventUserDeletedObjectIdentitiesItemPasswordlessIsSocialEnum,
        FieldMetadata(alias="isSocial"),
        pydantic.Field(alias="isSocial"),
    ]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionOptionsClientSecretTwitter = str
"""
OAuth 2.0 client secret issued by the identity provider during application registration. This value is used to authenticate your Auth0 connection to the identity provider.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .password_default_dictionaries_enum import PasswordDefaultDictionariesEnum


class ConnectionPasswordOptionsDictionary(UniversalBaseModel):
    """
    Dictionary-based password restriction policy to prevent common passwords
    """

    active: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enables dictionary checking to prevent use of common passwords and custom blocked words
    """

    custom: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Array of custom words to block in passwords. Maximum 200 items, each up to 50 characters
    """

    default: typing.Optional[PasswordDefaultDictionariesEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_user_created_object_app_metadata import EventStreamCloudEventUserCreatedObjectAppMetadata
from .event_stream_cloud_event_user_created_object_identities_item import (
    EventStreamCloudEventUserCreatedObjectIdentitiesItem,
)
from .event_stream_cloud_event_user_created_object_user_metadata import (
    EventStreamCloudEventUserCreatedObjectUserMetadata,
)


class EventStreamCloudEventUserCreatedObject(UniversalBaseModel):
    """
    The event content.
    """

    user_id: str = pydantic.Field()
    """
    ID of the user which can be used when interacting with other APIs.
    """

    email: typing.Optional[str] = pydantic.Field(default=None)
    """
    Email address of this user.
    """

    email_verified: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this email address is verified (true) or unverified (false).
    """

    username: typing.Optional[str] = pydantic.Field(default=None)
    """
    Username of this user.
    """

    phone_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Phone number of this user.
    """

    phone_verified: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this phone number has been verified (true) or not (false).
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Date and time when this entity was created (ISO_8601 format).
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Date and time when this entity was last updated/modified (ISO_8601 format).
    """

    identities: typing.List[EventStreamCloudEventUserCreatedObjectIdentitiesItem] = pydantic.Field()
    """
    Array of user identity objects when accounts are linked.
    """

    app_metadata: typing.Optional[EventStreamCloudEventUserCreatedObjectAppMetadata] = None
    user_metadata: typing.Optional[EventStreamCloudEventUserCreatedObjectUserMetadata] = None
    picture: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL to picture, photo, or avatar of this user.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this user.
    """

    nickname: typing.Optional[str] = pydantic.Field(default=None)
    """
    Preferred nickname or alias of this user.
    """

    multifactor: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of multi-factor authentication providers with which this user has enrolled.
    """

    last_ip: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last IP address from which this user logged in.
    """

    last_login: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Last date and time this user logged in (ISO_8601 format).
    """

    logins_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    Total number of logins this user has performed.
    """

    blocked: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether this user was blocked by an administrator (true) or is not (false).
    """

    given_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Given name/first name/forename of this user.
    """

    family_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Family name/last name/surname of this user.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .phone_template_content import PhoneTemplateContent
from .phone_template_notification_type_enum import PhoneTemplateNotificationTypeEnum


class UpdatePhoneTemplateResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = None
    channel: typing.Optional[str] = None
    customizable: typing.Optional[bool] = None
    tenant: typing.Optional[str] = None
    content: PhoneTemplateContent
    type: PhoneTemplateNotificationTypeEnum
    disabled: bool = pydantic.Field()
    """
    Whether the template is enabled (false) or disabled (true).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .credential_id import CredentialId

ClientAuthenticationMethodSelfSignedTlsClientAuthCredentials = typing.List[CredentialId]
"""
A list of unique and previously created credential IDs enabled on the client for mTLS authentication utilizing self-signed certificate.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .email_attribute_identifier import EmailAttributeIdentifier
from .signup_verified import SignupVerified
from .verification_method_enum import VerificationMethodEnum


class EmailAttribute(UniversalBaseModel):
    """
    Configuration for the email attribute for users.
    """

    identifier: typing.Optional[EmailAttributeIdentifier] = None
    unique: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Determines if the attribute is unique in a given connection
    """

    profile_required: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Determines if property should be required for users
    """

    verification_method: typing.Optional[VerificationMethodEnum] = None
    signup: typing.Optional[SignupVerified] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .connection_assertion_decryption_settings import ConnectionAssertionDecryptionSettings
from .connection_decryption_key_saml import ConnectionDecryptionKeySaml
from .connection_digest_algorithm_saml import ConnectionDigestAlgorithmSaml
from .connection_domain_aliases_saml import ConnectionDomainAliasesSaml
from .connection_entity_id_saml import ConnectionEntityIdSaml
from .connection_icon_url_saml import ConnectionIconUrlSaml
from .connection_options_idpinitiated_saml import ConnectionOptionsIdpinitiatedSaml
from .connection_protocol_binding_saml import ConnectionProtocolBindingSaml
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_sign_in_endpoint_saml import ConnectionSignInEndpointSaml
from .connection_sign_saml_request_saml import ConnectionSignSamlRequestSaml
from .connection_signature_algorithm_saml import ConnectionSignatureAlgorithmSaml
from .connection_signing_certificate_der_saml import ConnectionSigningCertificateDerSaml
from .connection_tenant_domain_saml import ConnectionTenantDomainSaml
from .connection_thumbprints_saml import ConnectionThumbprintsSaml
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsCommonSaml(UniversalBaseModel):
    """
    Common options for SAML-based enterprise connections (shared by samlp and pingfederate).
    """

    assertion_decryption_settings: typing.Optional[ConnectionAssertionDecryptionSettings] = None
    cert: typing.Optional[ConnectionSigningCertificateDerSaml] = None
    decryption_key: typing_extensions.Annotated[
        typing.Optional[ConnectionDecryptionKeySaml],
        FieldMetadata(alias="decryptionKey"),
        pydantic.Field(alias="decryptionKey"),
    ] = None
    digest_algorithm: typing_extensions.Annotated[
        typing.Optional[ConnectionDigestAlgorithmSaml],
        FieldMetadata(alias="digestAlgorithm"),
        pydantic.Field(alias="digestAlgorithm"),
    ] = None
    domain_aliases: typing.Optional[ConnectionDomainAliasesSaml] = None
    entity_id: typing_extensions.Annotated[
        typing.Optional[ConnectionEntityIdSaml], FieldMetadata(alias="entityId"), pydantic.Field(alias="entityId")
    ] = None
    icon_url: typing.Optional[ConnectionIconUrlSaml] = None
    idpinitiated: typing.Optional[ConnectionOptionsIdpinitiatedSaml] = None
    protocol_binding: typing_extensions.Annotated[
        typing.Optional[ConnectionProtocolBindingSaml],
        FieldMetadata(alias="protocolBinding"),
        pydantic.Field(alias="protocolBinding"),
    ] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    sign_in_endpoint: typing_extensions.Annotated[
        typing.Optional[ConnectionSignInEndpointSaml],
        FieldMetadata(alias="signInEndpoint"),
        pydantic.Field(alias="signInEndpoint"),
    ] = None
    sign_saml_request: typing_extensions.Annotated[
        typing.Optional[ConnectionSignSamlRequestSaml],
        FieldMetadata(alias="signSAMLRequest"),
        pydantic.Field(alias="signSAMLRequest"),
    ] = None
    signature_algorithm: typing_extensions.Annotated[
        typing.Optional[ConnectionSignatureAlgorithmSaml],
        FieldMetadata(alias="signatureAlgorithm"),
        pydantic.Field(alias="signatureAlgorithm"),
    ] = None
    tenant_domain: typing.Optional[ConnectionTenantDomainSaml] = None
    thumbprints: typing.Optional[ConnectionThumbprintsSaml] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionTenantDomainSaml = str
"""
For SAML connections, the tenant domain used to construct the login endpoint URL. Can be a string for single-tenant or an array of strings for multi-tenant validation.
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionAppDomainAzureAd = str
"""
The Azure AD application domain (e.g., 'contoso.onmicrosoft.com'). Used primarily with WS-Federation protocol and Azure AD v1 endpoints.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ClientAddonConcur = typing.Dict[str, typing.Any]
"""
Concur SSO indicator (no configuration settings needed for Concur SSO).
"""



from .conftest import get_client, verify_request_count


def test_rulesConfigs_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "rules_configs.list_.0"
    client = get_client(test_id)
    client.rules_configs.list()
    verify_request_count(test_id, "GET", "/rules-configs", None, 1)


def test_rulesConfigs_set_() -> None:
    """Test set endpoint with WireMock"""
    test_id = "rules_configs.set_.0"
    client = get_client(test_id)
    client.rules_configs.set(
        key="key",
        value="value",
    )
    verify_request_count(test_id, "PUT", "/rules-configs/key", None, 1)


def test_rulesConfigs_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "rules_configs.delete.0"
    client = get_client(test_id)
    client.rules_configs.delete(
        key="key",
    )
    verify_request_count(test_id, "DELETE", "/rules-configs/key", None, 1)



from .conftest import get_client, verify_request_count


def test_clientGrants_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "client_grants.list_.0"
    client = get_client(test_id)
    client.client_grants.list(
        from_="from",
        take=1,
        audience="audience",
        client_id="client_id",
        allow_any_organization=True,
        subject_type="client",
        default_for="third_party_clients",
    )
    verify_request_count(
        test_id,
        "GET",
        "/client-grants",
        {
            "from": "from",
            "take": "1",
            "audience": "audience",
            "client_id": "client_id",
            "allow_any_organization": "true",
            "subject_type": "client",
            "default_for": "third_party_clients",
        },
        1,
    )


def test_clientGrants_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "client_grants.create.0"
    client = get_client(test_id)
    client.client_grants.create(
        audience="audience",
    )
    verify_request_count(test_id, "POST", "/client-grants", None, 1)


def test_clientGrants_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "client_grants.get.0"
    client = get_client(test_id)
    client.client_grants.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/client-grants/id", None, 1)


def test_clientGrants_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "client_grants.delete.0"
    client = get_client(test_id)
    client.client_grants.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/client-grants/id", None, 1)


def test_clientGrants_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "client_grants.update.0"
    client = get_client(test_id)
    client.client_grants.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/client-grants/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_whatsapp_token import CreateFlowsVaultConnectionWhatsappToken
from .create_flows_vault_connection_whatsapp_uninitialized import CreateFlowsVaultConnectionWhatsappUninitialized

CreateFlowsVaultConnectionWhatsapp = typing.Union[
    CreateFlowsVaultConnectionWhatsappToken, CreateFlowsVaultConnectionWhatsappUninitialized
]



from .conftest import get_client, verify_request_count


def test_actions_modules_versions_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "actions.modules.versions.list_.0"
    client = get_client(test_id)
    client.actions.modules.versions.list(
        id="id",
        page=1,
        per_page=1,
    )
    verify_request_count(test_id, "GET", "/actions/modules/id/versions", {"page": "1", "per_page": "1"}, 1)


def test_actions_modules_versions_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "actions.modules.versions.create.0"
    client = get_client(test_id)
    client.actions.modules.versions.create(
        id="id",
    )
    verify_request_count(test_id, "POST", "/actions/modules/id/versions", None, 1)


def test_actions_modules_versions_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "actions.modules.versions.get.0"
    client = get_client(test_id)
    client.actions.modules.versions.get(
        id="id",
        version_id="versionId",
    )
    verify_request_count(test_id, "GET", "/actions/modules/id/versions/versionId", None, 1)



from .conftest import get_client, verify_request_count

from auth0.management import NetworkAclAction, NetworkAclRule


def test_networkAcls_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "network_acls.list_.0"
    client = get_client(test_id)
    client.network_acls.list(
        page=1,
        per_page=1,
        include_totals=True,
    )
    verify_request_count(test_id, "GET", "/network-acls", {"page": "1", "per_page": "1", "include_totals": "true"}, 1)


def test_networkAcls_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "network_acls.create.0"
    client = get_client(test_id)
    client.network_acls.create(
        description="description",
        active=True,
        rule=NetworkAclRule(
            action=NetworkAclAction(),
            scope="management",
        ),
    )
    verify_request_count(test_id, "POST", "/network-acls", None, 1)


def test_networkAcls_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "network_acls.get.0"
    client = get_client(test_id)
    client.network_acls.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/network-acls/id", None, 1)


def test_networkAcls_set_() -> None:
    """Test set endpoint with WireMock"""
    test_id = "network_acls.set_.0"
    client = get_client(test_id)
    client.network_acls.set(
        id="id",
        description="description",
        active=True,
        rule=NetworkAclRule(
            action=NetworkAclAction(),
            scope="management",
        ),
    )
    verify_request_count(test_id, "PUT", "/network-acls/id", None, 1)


def test_networkAcls_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "network_acls.delete.0"
    client = get_client(test_id)
    client.network_acls.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/network-acls/id", None, 1)


def test_networkAcls_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "network_acls.update.0"
    client = get_client(test_id)
    client.network_acls.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/network-acls/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .branding_theme_borders import BrandingThemeBorders
from .branding_theme_colors import BrandingThemeColors
from .branding_theme_fonts import BrandingThemeFonts
from .branding_theme_identifiers import BrandingThemeIdentifiers
from .branding_theme_page_background import BrandingThemePageBackground
from .branding_theme_widget import BrandingThemeWidget


class GetBrandingDefaultThemeResponseContent(UniversalBaseModel):
    borders: BrandingThemeBorders
    colors: BrandingThemeColors
    display_name: typing_extensions.Annotated[
        str, FieldMetadata(alias="displayName"), pydantic.Field(alias="displayName", description="Display Name")
    ]
    """
    Display Name
    """

    fonts: BrandingThemeFonts
    identifiers: typing.Optional[BrandingThemeIdentifiers] = None
    page_background: BrandingThemePageBackground
    theme_id: typing_extensions.Annotated[
        str, FieldMetadata(alias="themeId"), pydantic.Field(alias="themeId", description="Theme Id")
    ]
    """
    Theme Id
    """

    widget: BrandingThemeWidget

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .session_ip import SessionIp


class SessionDeviceMetadata(UniversalBaseModel):
    """
    Metadata related to the device used in the session
    """

    initial_user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    First user agent of the device from which this user logged in
    """

    initial_ip: typing.Optional[SessionIp] = None
    initial_asn: typing.Optional[str] = pydantic.Field(default=None)
    """
    First autonomous system number associated with this session
    """

    last_user_agent: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last user agent of the device from which this user logged in
    """

    last_ip: typing.Optional[SessionIp] = None
    last_asn: typing.Optional[str] = pydantic.Field(default=None)
    """
    Last autonomous system number from which this user logged in
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_custom_texts_by_language_response_content import GetCustomTextsByLanguageResponseContent
from ...types.prompt_group_name_enum import PromptGroupNameEnum
from ...types.prompt_language_enum import PromptLanguageEnum
from ...types.sets_custom_texts_by_language_request_content import SetsCustomTextsByLanguageRequestContent
from .raw_client import AsyncRawCustomTextClient, RawCustomTextClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CustomTextClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCustomTextClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCustomTextClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCustomTextClient
        """
        return self._raw_client

    def get(
        self,
        prompt: PromptGroupNameEnum,
        language: PromptLanguageEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCustomTextsByLanguageResponseContent:
        """
        Retrieve custom text for a specific prompt and language.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt.

        language : PromptLanguageEnum
            Language to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomTextsByLanguageResponseContent
            Prompt dictionaries successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.custom_text.get(
            prompt="login",
            language="am",
        )
        """
        _response = self._raw_client.get(prompt, language, request_options=request_options)
        return _response.data

    def set(
        self,
        prompt: PromptGroupNameEnum,
        language: PromptLanguageEnum,
        *,
        request: SetsCustomTextsByLanguageRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Set custom text for a specific prompt. Existing texts will be overwritten.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt.

        language : PromptLanguageEnum
            Language to update.

        request : SetsCustomTextsByLanguageRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.custom_text.set(
            prompt="login",
            language="am",
            request={"key": "value"},
        )
        """
        _response = self._raw_client.set(prompt, language, request=request, request_options=request_options)
        return _response.data


class AsyncCustomTextClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCustomTextClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCustomTextClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCustomTextClient
        """
        return self._raw_client

    async def get(
        self,
        prompt: PromptGroupNameEnum,
        language: PromptLanguageEnum,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetCustomTextsByLanguageResponseContent:
        """
        Retrieve custom text for a specific prompt and language.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt.

        language : PromptLanguageEnum
            Language to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetCustomTextsByLanguageResponseContent
            Prompt dictionaries successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.custom_text.get(
                prompt="login",
                language="am",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(prompt, language, request_options=request_options)
        return _response.data

    async def set(
        self,
        prompt: PromptGroupNameEnum,
        language: PromptLanguageEnum,
        *,
        request: SetsCustomTextsByLanguageRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Set custom text for a specific prompt. Existing texts will be overwritten.

        Parameters
        ----------
        prompt : PromptGroupNameEnum
            Name of the prompt.

        language : PromptLanguageEnum
            Language to update.

        request : SetsCustomTextsByLanguageRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.custom_text.set(
                prompt="login",
                language="am",
                request={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(prompt, language, request=request, request_options=request_options)
        return _response.data
